import _ from 'lodash';
import {compose} from 'recompose';
import configureStore from 'redux-mock-store';
import {hoc} from '../';
import {mount} from 'enzyme';
import thunk from 'redux-thunk';
jest.mock('whatwg-fetch');

describe('Details HOC', () => {
	let
		store;

	beforeEach(() => { store = configureStore([thunk])({}); });

	it('syncs remote data with redux store being applied with [fetchOnMount === true]', () => {
		mount(hoc.details({
			baseURL     : '/',
			entity      : 'contact',
			fetchOnMount: true,
			key         : 'contact',
			urlParams   : {id: '1a'},
		})(() => null)({store}));

		expect(store.getActions().pop()).toEqual({
			request: {params: {}, pathvars: {}},
			syncing: true,
			type   : '@@redux-api@contact',
		});
	});

	it('doesn\'t sync remote data with redux store being applied with [fetchOnMount === false]', () => {
		mount(hoc.details({
			baseURL     : '/',
			entity      : 'contact',
			fetchOnMount: false,
			key         : 'contact',
			urlParams   : {id: '1a'},
		})(() => null)({store}));

		expect(store.getActions().pop()).toEqual(undefined);
	});

	it('syncs remote data on [queryParams] change', () => {
		const
			instance = mount(hoc.details({
				baseURL    : '/',
				entity     : 'contact',
				key        : 'contact',
				queryParams: props => ({testParam: props.testProp}),
			})(() => null)({store, testProp: 'a'}));

		instance.setProps({store, testProp: 'b'});

		expect(store.getActions().pop()).toEqual({
			request: {params: {}, pathvars: {testParam: 'b'}},
			syncing: true,
			type   : '@@redux-api@contact',
		});
	});

	it('syncs remote data on [urlParams] change', () => {
		const
			instance = mount(hoc.details({
				baseURL  : '/',
				entity   : 'contact',
				key      : 'contact',
				urlParams: props => ({id: props.id}),
			})(() => null)({id: 'a', store}));

		instance.setProps({id: 'b', store});

		expect(store.getActions().pop()).toEqual({
			request: {params: {}, pathvars: {id: 'b'}},
			syncing: true,
			type   : '@@redux-api@contact',
		});
	});

	it('provides delete(), post() and put() methods to a wrapped component', () => {
		mount(hoc.details({
			baseURL  : 'http://localhost/test',
			entity   : 'contact',
			key      : 'contact',
			urlParams: props => ({id: props.id}),
		})(props => {
			props.faFetched.contact.put({id: 1}, {body: '{"test": "body"}'});

			expect(_.last(store.getActions())).toEqual({
				request: {params: {body: '{"test": "body"}', method: 'PUT'}, pathvars: {id: 1}},
				syncing: false,
				type   : '@@redux-api@contact',
			});

			props.faFetched.contact.post({body: '{"test": "body"}'});

			expect(_.last(store.getActions())).toEqual({
				request: {params: {method: 'POST'}, pathvars: {body: '{"test": "body"}'}},
				syncing: false,
				type   : '@@redux-api@contact',
			});

			props.faFetched.contact.delete({id: '1'});

			expect(_.last(store.getActions())).toEqual({
				request: {params: {method: 'DELETE'}, pathvars: {id: '1'}},
				syncing: false,
				type   : '@@redux-api@contact',
			});

			return null;
		})({id: 'a', store}));
	});

	it('provides delete(), post() and put() methods to a component wrapped with FetchAPI multiple times', () => {
		mount(compose(
			hoc.details({
				baseURL: '/',
				entity : 'contacts',
				key    : 'contacts',
			}),

			hoc.details({
				baseURL  : '/',
				entity   : 'contact',
				key      : 'contact',
				urlParams: props => ({id: props.id}),
			}),
		)(props => {
			expect(_.isFunction(props.faFetched.contact.delete)).toBeTruthy();
			expect(_.isFunction(props.faFetched.contact.post)).toBeTruthy();
			expect(_.isFunction(props.faFetched.contact.put)).toBeTruthy();
			return null;
		})({id: 'a', store}));

		mount(compose(
			hoc.details({
				baseURL  : '/',
				entity   : 'contact',
				key      : 'contact',
				urlParams: props => ({id: props.id}),
			}),

			hoc.details({
				baseURL: '/',
				entity : 'contacts',
				key    : 'contacts',
			}),
		)(props => {
			expect(_.isFunction(props.faFetched.contact.delete)).toBeTruthy();
			expect(_.isFunction(props.faFetched.contact.post)).toBeTruthy();
			expect(_.isFunction(props.faFetched.contact.put)).toBeTruthy();
			return null;
		})({id: 'a', store}));
	});
});
